(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('date-fns/locale'), require('date-fns')) :
    typeof define === 'function' && define.amd ? define(['exports', 'react', 'date-fns/locale', 'date-fns'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.DayPicker = {}, global.React, global.locale, global.DateFns));
}(this, (function (exports, React, locale, dateFns) { 'use strict';

    function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

    var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */

    var __assign = function() {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };

    /** @deprecated */
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }

    /** Reference to keyboard event codes used by DayPicker. */
    exports.KeyCode = void 0;
    (function (KeyCode) {
        KeyCode["ArrowLeft"] = "ArrowLeft";
        KeyCode["ArrowRight"] = "ArrowRight";
        KeyCode["ArrowUp"] = "ArrowUp";
        KeyCode["ArrowDown"] = "ArrowDown";
        KeyCode["Space"] = "Space";
        KeyCode["Enter"] = "Enter";
    })(exports.KeyCode || (exports.KeyCode = {}));

    /**
     * Represent an element or an element state of the DayPicker user interface,
     * whose style or class name can be changed using the `styles` or `classNames`
     * prop.
     *
     * ```
     * <DayPicker
     *   classNames={{ captionLast: 'my-custom-class' }}
     *   styles={{ captionLast: { color: 'red' }}}
     * />
     * ```
     */
    exports.UIElement = void 0;
    (function (UIElement) {
        /** The container element. */
        UIElement["Root"] = "root";
        /** The container element when number of months is greater than 1. */
        UIElement["RootMultipleMonths"] = "rootMultipleMonths";
        /** The caption element (showing the calendar heading and the navigation) */
        UIElement["Caption"] = "caption";
        /** The caption element when the first of multiple months */
        UIElement["CaptionFirst"] = "captionFirst";
        /** The caption element when the last of multiple months */
        UIElement["CaptionLast"] = "captionLast";
        /** The caption element is between */
        UIElement["CaptionBetween"] = "captionBetween";
        /** The container of the caption’s label (e.g. "September 2021"). */
        UIElement["CaptionLabel"] = "captionLabel";
        /** The container of the drop-downs. */
        UIElement["CaptionDropdowns"] = "dropdownsContainer";
        /** The drop-down select element. */
        UIElement["Dropdown"] = "dropdown";
        /** The drop-down to change the month. */
        UIElement["DropdownMonth"] = "dropdownMonth";
        /** The drop-down to change the year. */
        UIElement["DropdownYear"] = "dropdownYear";
        /** The icon aside the drop-down. */
        UIElement["DropdownIcon"] = "dropdownIcon";
        /** The wrapper of the months (there may be more months visualized when `numberOfMonths`). */
        UIElement["Months"] = "months";
        /** The wrapper of the table displaying the month. */
        UIElement["Month"] = "month";
        /** The table displaying the calendar. */
        UIElement["Table"] = "table";
        /** The body element of the table displaying the calendar. */
        UIElement["TBody"] = "body";
        /** The head element of the table displaying the calendar. */
        UIElement["Head"] = "head";
        /** The row element of the head. */
        UIElement["HeadRow"] = "headRow";
        /** The cell element of the head. */
        UIElement["HeadCell"] = "headCell";
        /** The navigation element. */
        UIElement["Nav"] = "nav";
        /** A navigation button. */
        UIElement["NavButton"] = "navButton";
        /** The "previous month" navigation button. */
        UIElement["NavButtonPrev"] = "navButtonPrev";
        /** The "next month" navigation button. */
        UIElement["NavButtonNext"] = "navButtonNext";
        /** The icon contained in navigation button. */
        UIElement["NavIcon"] = "navIcon";
        /** A row in the table element (each row shows a week). */
        UIElement["Row"] = "row";
        /** A row head in the table element, used to display the week numbers. */
        UIElement["RowHead"] = "rowHead";
        /** The element containing the week number */
        UIElement["WeekNumber"] = "weekNumber";
        /** The cell containing the day element. */
        UIElement["Cell"] = "cell";
        /** The day element. */
        UIElement["Day"] = "day";
        /** The day element when outside the month */
        UIElement["Outside"] = "outside";
        /** The day element when is today */
        UIElement["Today"] = "today";
    })(exports.UIElement || (exports.UIElement = {}));

    var _a;
    /**
     * The name of the default CSS classes for the [[UIElements]].
     */
    var defaultClassNames = (_a = {},
        /** Root element */
        _a[exports.UIElement.Root] = 'rdp',
        _a[exports.UIElement.RootMultipleMonths] = 'rdp-multiple-months',
        _a[exports.UIElement.Months] = 'rdp-months',
        _a[exports.UIElement.CaptionFirst] = 'rdp-month_first',
        _a[exports.UIElement.CaptionLast] = 'rdp-month_last',
        _a[exports.UIElement.CaptionBetween] = 'rdp-month_between',
        _a[exports.UIElement.Caption] = 'rdp-caption',
        _a[exports.UIElement.CaptionLabel] = 'rdp-caption-label',
        _a[exports.UIElement.CaptionDropdowns] = 'rdp-caption-dropdowns',
        _a[exports.UIElement.Dropdown] = 'rdp-dropdown',
        _a[exports.UIElement.DropdownMonth] = 'rdp-dropdown_month',
        _a[exports.UIElement.DropdownYear] = 'rdp-dropdown_year',
        // Day Component
        _a[exports.UIElement.Day] = 'rdp-day',
        _a[exports.UIElement.Outside] = 'rdp-day_outside',
        _a[exports.UIElement.Today] = 'rdp-day_today',
        // Month Component
        _a[exports.UIElement.Month] = 'rdp-month',
        _a[exports.UIElement.Table] = 'rdp-table',
        _a[exports.UIElement.TBody] = 'rdp-body',
        // Head Components
        _a[exports.UIElement.Head] = 'rdp-head',
        _a[exports.UIElement.HeadRow] = 'rdp-head-row',
        _a[exports.UIElement.HeadCell] = 'rdp-head-cell',
        // Navigation Component
        _a[exports.UIElement.Nav] = 'rdp-nav',
        _a[exports.UIElement.NavButton] = 'rdp-nav-button',
        _a[exports.UIElement.NavButtonPrev] = 'rdp-nav-button_prev',
        _a[exports.UIElement.NavButtonNext] = 'rdp-nav-button_next',
        _a[exports.UIElement.NavIcon] = 'rdp-nav-icon',
        _a[exports.UIElement.DropdownIcon] = 'rdp-dropdown-icon',
        // Week Component
        _a[exports.UIElement.Row] = 'rdp-row',
        _a[exports.UIElement.RowHead] = 'rdp-row-head',
        _a[exports.UIElement.WeekNumber] = 'rdp-weeknumber',
        _a[exports.UIElement.Cell] = 'rdp-cell',
        _a);

    /**
     * The default formatter for the caption.
     */
    function formatCaption(month, options) {
        return dateFns.format(month, 'LLLL y', options);
    }

    /**
     * The default formatter for the Day button.
     */
    function formatDay(day, options) {
        return dateFns.format(day, 'd', options);
    }

    /**
     * The default formatter for the Month caption.
     */
    function formatMonthCaption(month, options) {
        return dateFns.format(month, 'LLLL', options);
    }

    /**
     * The default formatter for the week number.
     */
    function formatWeekNumber(weekNumber) {
        return "" + weekNumber;
    }

    /**
     * The default formatter for the name of the weekday.
     */
    function formatWeekdayName(weekday, options) {
        return dateFns.format(weekday, 'cccccc', options);
    }

    /**
     * The default formatter for the Year caption.
     */
    function formatYearCaption(year, options) {
        return dateFns.format(year, 'yyyy', options);
    }

    var formatters = /*#__PURE__*/Object.freeze({
        __proto__: null,
        formatCaption: formatCaption,
        formatDay: formatDay,
        formatMonthCaption: formatMonthCaption,
        formatWeekNumber: formatWeekNumber,
        formatWeekdayName: formatWeekdayName,
        formatYearCaption: formatYearCaption
    });

    /**
     * The default ARIA label for the day button.
     */
    var labelDay = function (day, options) {
        return dateFns.format(day, 'PPPP', options);
    };

    /**
     * The default ARIA label for next month button in navigation
     */
    var labelNext = function (month, options) {
        return "Go to next month: " + dateFns.format(month, 'LLLL Y', options);
    };

    /**
     * The default ARIA label for previous month button in navigation
     */
    var labelPrevious = function (month, options) {
        return "Go to previous month: " + dateFns.format(month, 'LLLL Y', options);
    };

    /**
     * The default ARIA label for the WeekNumber element.
     */
    var labelWeekNumber = function (n) {
        return "Week n. " + n;
    };

    /**
     * The default ARIA label for the Weekday element.
     */
    var labelWeekday = function (day, options) {
        return dateFns.format(day, 'ccc', options);
    };

    var labels = /*#__PURE__*/Object.freeze({
        __proto__: null,
        labelDay: labelDay,
        labelNext: labelNext,
        labelPrevious: labelPrevious,
        labelWeekNumber: labelWeekNumber,
        labelWeekday: labelWeekday
    });

    /**
     * Generate a series of 7 days, starting from the week, to use for formatting
     * the weekday names (Monday, Tuesday, etc.).
     */
    function getWeekdays(locale) {
        var start = dateFns.startOfWeek(new Date(), { locale: locale });
        var days = [];
        for (var i = 0; i < 7; i++) {
            var day = dateFns.addDays(start, i);
            days.push(day);
        }
        return days;
    }

    /**
     * Build the `fromDate` and `toDate` context values, given the fromMonth/toMonth
     * or fromYear/toYear.
     */
    function parseFromToProps(props) {
        var fromYear = props.fromYear, toYear = props.toYear, fromMonth = props.fromMonth, toMonth = props.toMonth;
        var fromDate = props.fromDate, toDate = props.toDate;
        if (fromMonth) {
            fromDate = dateFns.startOfMonth(fromMonth);
        }
        else if (fromYear) {
            fromDate = new Date(fromYear, 0, 1);
        }
        if (toMonth) {
            toDate = dateFns.startOfMonth(toMonth);
        }
        else if (toYear) {
            toDate = new Date(toYear, 11, 31);
        }
        return { fromDate: fromDate, toDate: toDate };
    }

    /**
     * The DayPickerContext is a context to share props and settings within the
     * internal components. It set the defaults values, parses some props, and
     * perform one-time calculation required to render the days.
     */
    var DayPickerContext = React.createContext(undefined);
    /**
     * The provider for the [[DayPickerContext]]. Must wrap the DayPicker’s root.
     */
    var DayPickerProvider = function (_a) {
        var _b, _c, _d, _e, _f, _g;
        var initialProps = _a.initialProps, children = _a.children;
        var _h = parseFromToProps(initialProps), fromDate = _h.fromDate, toDate = _h.toDate;
        var locale$1 = initialProps.locale || locale.enUS;
        var numberOfMonths = (_b = initialProps.numberOfMonths) !== null && _b !== void 0 ? _b : 1;
        var today = (_c = initialProps.today) !== null && _c !== void 0 ? _c : new Date();
        var month = initialProps.month;
        var weekdays = getWeekdays(locale$1);
        // Default selection mode. If using `selected` prop, it is uncontrolled
        var mode = (_d = initialProps.mode) !== null && _d !== void 0 ? _d : 'single';
        if (initialProps.selected)
            mode = 'uncontrolled';
        // Default caption layout. If calendar navigation is unlimited it must be
        // always `buttons` as we cannot display infinite options in the dropdown.
        var captionLayout = (_e = initialProps.captionLayout) !== null && _e !== void 0 ? _e : 'buttons';
        if (!fromDate && !toDate)
            captionLayout = 'buttons';
        // Set defaults from modifiers shortcuts props
        var modifiers = initialProps.modifiers || {};
        if (initialProps.selected)
            modifiers.selected = initialProps.selected;
        if (initialProps.hidden)
            modifiers.hidden = initialProps.hidden;
        modifiers.disabled = [];
        if (initialProps.disabled) {
            if (Array.isArray(initialProps.disabled))
                modifiers.disabled = __spreadArrays(initialProps.disabled);
            else
                modifiers.disabled = [initialProps.disabled];
        }
        // Disable days before/after from/toDate
        if (fromDate)
            modifiers.disabled.push({ before: fromDate });
        if (toDate)
            modifiers.disabled.push({ after: toDate });
        var context = __assign(__assign({}, initialProps), { modifierPrefix: 'rdp-day_', numberOfMonths: numberOfMonths,
            fromDate: fromDate,
            toDate: toDate,
            captionLayout: captionLayout,
            mode: mode,
            month: month,
            today: today,
            locale: locale$1,
            weekdays: weekdays, modifierClassNames: (_f = initialProps.modifierClassNames) !== null && _f !== void 0 ? _f : {}, styles: (_g = initialProps.styles) !== null && _g !== void 0 ? _g : {}, modifiers: modifiers, classNames: __assign(__assign({}, defaultClassNames), initialProps.classNames), formatters: __assign(__assign({}, formatters), initialProps.formatters), labels: __assign(__assign({}, labels), initialProps.labels), components: __assign({ Caption: Caption, CaptionLabel: CaptionLabel, Day: Day, DayContent: DayContent, Dropdown: Dropdown, Footer: Footer, Head: Head, IconDropdown: IconDropdown, IconNext: IconNext, IconPrevious: IconPrevious, Row: Row, WeekNumber: WeekNumber }, initialProps.components) });
        return (React.createElement(DayPickerContext.Provider, { value: context }, children));
    };

    /**
     * Return the [[DayPickerContext]].
     *
     * Use the DayPickerContext to get the props and other values directly from the
     * internal components.
     */
    function useDayPicker() {
        var context = React__default['default'].useContext(DayPickerContext);
        if (!context) {
            throw new Error('useDayPickerContext must be used within a DayPickerProvider');
        }
        return context;
    }

    /** Hook to handle multiple selections. */
    function useMultipleSelect(mode, defaultValue, required, callback) {
        if (required === void 0) { required = false; }
        var initialValue = mode === 'multiple' && defaultValue ? defaultValue : [];
        var _a = React.useState(initialValue), selected = _a[0], setSelected = _a[1];
        var handleClick = function (day, modifiers, e) {
            setSelected(function (currentValue) {
                var newValue = __spreadArrays(currentValue);
                if (modifiers.selected) {
                    if (required && currentValue.length === 1) {
                        return newValue;
                    }
                    var index = currentValue.findIndex(function (value) { return dateFns.isSameDay(day, value); });
                    newValue.splice(index, 1);
                }
                else {
                    newValue.push(day);
                }
                callback === null || callback === void 0 ? void 0 : callback(newValue, day, modifiers, e);
                return newValue;
            });
        };
        var reset = function () {
            setSelected(initialValue);
        };
        return { selected: selected, onDayClick: handleClick, reset: reset };
    }

    /**
     * Add a day to an existing range.
     *
     * The returned range takes in account the `undefined` values and if the added
     * day is already present in the range.
     */
    function addToRange(day, range, required) {
        if (required === void 0) { required = false; }
        var _a = range || {}, from = _a.from, to = _a.to;
        if (!from) {
            return { from: day };
        }
        if (!required && !to && dateFns.isSameDay(from, day)) {
            return undefined;
        }
        if (!to && dateFns.isBefore(day, from)) {
            return { from: day, to: from };
        }
        if (!to) {
            return { from: from, to: day };
        }
        if (dateFns.isSameDay(to, day)) {
            return { from: to };
        }
        if (dateFns.isAfter(from, day)) {
            return { from: day, to: to };
        }
        return { from: from, to: day };
    }

    /** Hook to handle range selections. */
    function useRangeSelect(mode, defaultValue, required, callback) {
        if (required === void 0) { required = false; }
        var initialValue = mode === 'range' && defaultValue ? defaultValue : undefined;
        var _a = React.useState(initialValue), selected = _a[0], setSelected = _a[1];
        var onDayClick = function (day, modifiers, e) {
            setSelected(function (currentValue) {
                var newValue = addToRange(day, currentValue, required);
                callback === null || callback === void 0 ? void 0 : callback(newValue, day, modifiers, e);
                setSelected(newValue);
                return newValue;
            });
            return;
        };
        var reset = function () {
            setSelected(initialValue || undefined);
        };
        return { selected: selected, onDayClick: onDayClick, reset: reset };
    }

    /** Hook to handle single selections. */
    function useSingleSelect(mode, defaultValue, required, callback) {
        if (required === void 0) { required = false; }
        var initialValue = mode === 'single' && defaultValue ? defaultValue : undefined;
        var _a = React.useState(initialValue), selected = _a[0], setSelected = _a[1];
        var onDayClick = function (day, modifiers, e) {
            if (modifiers.selected) {
                if (!required) {
                    setSelected(undefined);
                    callback === null || callback === void 0 ? void 0 : callback(undefined, day, modifiers, e);
                }
                return;
            }
            setSelected(day);
            callback === null || callback === void 0 ? void 0 : callback(day, day, modifiers, e);
        };
        var reset = function () {
            setSelected(initialValue);
        };
        return { selected: selected, onDayClick: onDayClick, reset: reset };
    }

    var SelectionContext = React.createContext(undefined);
    /**
     * Provides the setters and values for the controlled selection modes.
     */
    var SelectionProvider = function (props) {
        var dayPickerProps = useDayPicker();
        var single = useSingleSelect('single', dayPickerProps.defaultSelected, dayPickerProps.required, dayPickerProps.onSelect);
        var multiple = useMultipleSelect('multiple', dayPickerProps.defaultSelected, dayPickerProps.required, dayPickerProps.onSelectMultiple);
        var range = useRangeSelect('range', dayPickerProps.defaultSelected, dayPickerProps.required, dayPickerProps.onSelectRange);
        return (React.createElement(SelectionContext.Provider, { value: { single: single, multiple: multiple, range: range } }, props.children));
    };

    /** Return the context for the controlled mode selection. */
    function useSelection() {
        var context = React__default['default'].useContext(SelectionContext);
        if (!context) {
            throw new Error('useSelection must be used within a SelectionProvider');
        }
        return context;
    }

    function matchAfter(day, matcher) {
        if (!('after' in matcher) || 'before' in matcher)
            return false;
        return dateFns.differenceInCalendarDays(day, matcher.after) > 0;
    }

    function matchBefore(day, matcher) {
        if (!('before' in matcher) || 'after' in matcher)
            return false;
        return dateFns.differenceInCalendarDays(day, matcher.before) < 0;
    }

    function matchBeforeAndAfter(day, matcher) {
        if (!('before' in matcher) || !('after' in matcher))
            return false;
        return (dateFns.differenceInCalendarDays(day, matcher.before) < 0 &&
            dateFns.differenceInCalendarDays(day, matcher.after) > 0);
    }

    function matchDate(day, matcher) {
        if (!(matcher instanceof Date))
            return false;
        return dateFns.isSameDay(day, matcher);
    }

    function matchDaysOfWeek(day, matcher) {
        if (!('daysOfWeek' in matcher))
            return false;
        return matcher.daysOfWeek.includes(day.getDay());
    }

    function matchFunction(day, matcher) {
        if (!(matcher instanceof Function))
            return false;
        return matcher(day);
    }

    function matchRange(day, matcher) {
        var _a;
        if (!('from' in matcher))
            return false;
        if (!matcher.from)
            return false;
        var from = matcher.from, to = matcher.to;
        // Matches { from: <Date> }
        if (!to && from && dateFns.isSameDay(from, day))
            return true;
        if (!to)
            return false;
        // Invert the case where "to" is before "from"
        if (matcher.to && dateFns.differenceInCalendarDays(matcher.to, matcher.from) < 0) {
            _a = [matcher.to, matcher.from], from = _a[0], to = _a[1];
        }
        return (dateFns.differenceInCalendarDays(day, from) >= 0 &&
            dateFns.differenceInCalendarDays(to, day) >= 0);
    }

    /**
     * Returns true when the day matches against the given matcher. Supports few
     * basic matchers.
     */
    function isMatch(day, matcher) {
        if (!matcher)
            return false;
        var matchers;
        if (Array.isArray(matcher)) {
            matchers = matcher;
        }
        else {
            matchers = [matcher];
        }
        return matchers.some(function (dayMatcher) {
            if (!dayMatcher)
                return false;
            return (
            // Precedence shouldn't be important here
            matchDate(day, dayMatcher) ||
                matchBeforeAndAfter(day, dayMatcher) ||
                matchBefore(day, dayMatcher) ||
                matchAfter(day, dayMatcher) ||
                matchRange(day, dayMatcher) ||
                matchDaysOfWeek(day, dayMatcher) ||
                matchFunction(day, dayMatcher));
        });
    }

    /**
     * Return the status of the modifiers for the given day,
     */
    function getModifierStatus(day, 
    /** The modifiers for the given day. */
    modifiers) {
        var modifiersList = Object.keys(modifiers).reduce(function (previousValue, key) {
            var modifier = modifiers[key];
            if (isMatch(day, modifier)) {
                previousValue.push(key);
            }
            return previousValue;
        }, []);
        var modifiersStatus = {};
        modifiersList.forEach(function (modifier) { return (modifiersStatus[modifier] = true); });
        return modifiersStatus;
    }

    /** Return the status for the modifiers given the specified date */
    function useModifiers(date) {
        var context = useDayPicker();
        var selection = useSelection();
        var modifiers = Object.assign({}, context.modifiers);
        modifiers.today = context.today;
        if (context.mode !== 'uncontrolled') {
            var _a = selection !== null && selection !== void 0 ? selection : {}, single = _a.single, multiple = _a.multiple, range = _a.range;
            switch (context.mode) {
                case 'range':
                    if (!range.selected)
                        break;
                    modifiers.selected = range.selected;
                    modifiers['range-middle'] = {
                        after: range.selected.from,
                        before: range.selected.to
                    };
                    modifiers['range-start'] = range.selected.from;
                    if (range.selected.to)
                        modifiers['range-end'] = range.selected.to;
                    break;
                case 'multiple':
                    if (!multiple.selected)
                        break;
                    modifiers.selected = multiple.selected;
                    break;
                default:
                case 'single':
                    if (!single.selected)
                        break;
                    modifiers.selected = single.selected;
                    break;
            }
        }
        var status = getModifierStatus(date, modifiers);
        return status;
    }

    /** @private */
    function isValidDate(day) {
        return !isNaN(day.getTime());
    }
    /**
     * Return the props for binding an input field with DayPicker.
     *
     * **Example**
     *
     * ```
     * function Example() {
     *   const { dayPickerProps, inputProps } = useInput(new Date(), 'yyyy-MM-dd');
     *   return (
     *     <>
     *       <p>
     *         Type a day or pick one from the calendar:
     *         <input {...inputProps} placeholder="YYYY-MM-DD" />
     *       </p>
     *       <DayPicker {...dayPickerProps} />
     *     </>
     *   );
     * }
     * ```
     */
    function useInput(initialDay, formatStr, options) {
        var _a;
        // Defaults from options
        var locale = options === null || options === void 0 ? void 0 : options.locale;
        var required = (_a = options === null || options === void 0 ? void 0 : options.required) !== null && _a !== void 0 ? _a : false;
        // Shortcut to the DateFns functions
        var formatDay = function (day) { return dateFns.format(day, formatStr, { locale: locale }); };
        var parseValue = function (value) {
            return dateFns.parse(value, formatStr, new Date(), { locale: locale });
        };
        // The initial value of the input field
        var initialValue = initialDay ? formatDay(initialDay) : '';
        // Initialize state
        var _b = React.useState(initialDay), selected = _b[0], setSelected = _b[1];
        var _c = React.useState(initialValue), value = _c[0], setValue = _c[1];
        var _d = React.useState(initialDay !== null && initialDay !== void 0 ? initialDay : new Date()), month = _d[0], setMonth = _d[1];
        var onDayClick = function (day) {
            setSelected(day);
            setValue(formatDay(day));
        };
        var onMonthChange = function (month) {
            setMonth(month);
        };
        // When changing the input field, save its value in state and check if the
        // string is a valid date. If it is a valid day, set it as selected and update
        // the calendar’s month.
        var onChange = function (e) {
            setValue(e.target.value);
            var day = parseValue(e.target.value);
            if (!isValidDate(day)) {
                setSelected(undefined);
                return;
            }
            setSelected(day);
            setMonth(day);
        };
        // Special case for _required_ fields: on blur, if the value of the input is not
        // a valid date, reset the calendar and the input value.
        var onBlur = function (e) {
            var _a;
            var day = parseValue(e.target.value);
            if ((_a = !required) !== null && _a !== void 0 ? _a : isValidDate(day)) {
                return;
            }
            setSelected(initialDay);
            setMonth(initialDay !== null && initialDay !== void 0 ? initialDay : new Date());
            setValue(initialValue !== null && initialValue !== void 0 ? initialValue : '');
        };
        // When focusing, make sure DayPicker visualizes the month of the date in the
        // input field.
        var onFocus = function (e) {
            if (e.target.value) {
                var day = parseValue(e.target.value);
                if (isValidDate(day)) {
                    setMonth(day);
                }
                return;
            }
            setSelected(initialDay);
            setMonth(initialDay !== null && initialDay !== void 0 ? initialDay : new Date());
            setValue(initialValue !== null && initialValue !== void 0 ? initialValue : '');
        };
        return {
            setMonth: setMonth,
            setValue: setValue,
            dayPickerProps: {
                month: month,
                selected: selected,
                onDayClick: onDayClick,
                onMonthChange: onMonthChange,
                mode: 'uncontrolled'
            },
            inputProps: {
                value: value,
                onChange: onChange,
                onFocus: onFocus,
                onBlur: onBlur
            }
        };
    }

    /**
     * Return the months to display in the component according to the number of
     * months and the from/to date.
     */
    function getDisplayMonths(month, _a) {
        var reverseMonths = _a.reverseMonths, numberOfMonths = _a.numberOfMonths;
        var start = dateFns.startOfMonth(month);
        var end = dateFns.startOfMonth(dateFns.addMonths(start, numberOfMonths));
        var monthsDiff = dateFns.differenceInCalendarMonths(end, start);
        var months = [];
        for (var i = 0; i < monthsDiff; i++) {
            var nextMonth = dateFns.addMonths(start, i);
            months.push(nextMonth);
        }
        if (reverseMonths)
            months = months.reverse();
        return months;
    }

    function getInitialMonth(context) {
        var month = context.month, defaultMonth = context.defaultMonth, _a = context.today, today = _a === void 0 ? new Date() : _a;
        var initialMonth = month || defaultMonth || today;
        var toDate = context.toDate, fromDate = context.fromDate, _b = context.numberOfMonths, numberOfMonths = _b === void 0 ? 1 : _b;
        // Fix the initialMonth if is after the to-date
        if (toDate && dateFns.differenceInCalendarMonths(toDate, initialMonth) < 0) {
            var offset = -1 * (numberOfMonths - 1);
            initialMonth = dateFns.addMonths(toDate, offset);
        }
        // Fix the initialMonth if is before the from-date
        if (fromDate && dateFns.differenceInCalendarMonths(initialMonth, fromDate) < 0) {
            initialMonth = fromDate;
        }
        return dateFns.startOfMonth(initialMonth);
    }

    /**
     * Returns the next month the user can navigate to according to the given
     * options.
     *
     * The next month is not always the next calendar month:
     *
     * - if after the `toDate` range, is undefined
     * - if the navigation is paged, is the number of months displayed ahead
     */
    function getNextMonth(startingMonth, options) {
        if (options.disableNavigation) {
            return undefined;
        }
        var toDate = options.toDate, pagedNavigation = options.pagedNavigation, _a = options.numberOfMonths, numberOfMonths = _a === void 0 ? 1 : _a;
        var offset = pagedNavigation ? numberOfMonths : 1;
        var month = dateFns.startOfMonth(startingMonth);
        if (!toDate) {
            return dateFns.addMonths(month, offset);
        }
        var monthsDiff = dateFns.differenceInCalendarMonths(toDate, startingMonth);
        if (monthsDiff < numberOfMonths) {
            return undefined;
        }
        // Jump forward as the number of months when paged navigation
        return dateFns.addMonths(month, offset);
    }

    /**
     * Returns the next previous the user can navigate to, according to the given
     * options.
     *
     * The previous month is not always the previous calendar month:
     *
     * - if before the `fromDate` date, is `undefined`
     * - if the navigation is paged, is the number of months displayed before
     */
    function getPreviousMonth(startingMonth, options) {
        if (options.disableNavigation) {
            return undefined;
        }
        var fromDate = options.fromDate, pagedNavigation = options.pagedNavigation, _a = options.numberOfMonths, numberOfMonths = _a === void 0 ? 1 : _a;
        var offset = pagedNavigation ? numberOfMonths : 1;
        var month = dateFns.startOfMonth(startingMonth);
        if (!fromDate) {
            return dateFns.addMonths(month, -offset);
        }
        var monthsDiff = dateFns.differenceInCalendarMonths(month, fromDate);
        if (monthsDiff <= 0) {
            return undefined;
        }
        // Jump back as the number of months when paged navigation
        return dateFns.addMonths(month, -offset);
    }

    /** The navigation context holds values and setters for navigating between the
     * calendars. */
    var NavigationContext = React.createContext(undefined);
    /** Provides the values for the [[NavigationContext]]. */
    var NavigationProvider = function (_a) {
        var children = _a.children;
        var context = useDayPicker();
        var initialMonth = getInitialMonth(context);
        var _b = React.useState(initialMonth), month = _b[0], setMonthInternal = _b[1];
        var setMonth = function (date) {
            if (context.disableNavigation)
                return;
            setMonthInternal(date);
        };
        React.useEffect(function () {
            if (!context.month)
                return;
            if (dateFns.isSameMonth(context.month, month))
                return;
            setMonth(context.month);
        }, [context.month]);
        var displayMonths = getDisplayMonths(month, context);
        var nextMonth = getNextMonth(month, context);
        var previousMonth = getPreviousMonth(month, context);
        return (React.createElement(NavigationContext.Provider, { value: { month: month, displayMonths: displayMonths, setMonth: setMonth, previousMonth: previousMonth, nextMonth: nextMonth } }, children));
    };

    /** Return the values for handing the navigation between months. */
    function useNavigation(currentMonth) {
        var context = React.useContext(NavigationContext);
        if (!context) {
            throw new Error('useNavigation must be used within a NavigationProvider');
        }
        return context;
    }

    /**
     * Render the caption of a month, which includes title and navigation buttons.
     * The caption has a different layout when setting the `numberOfMonths` prop.
     */
    function Caption(props) {
        var displayMonth = props.displayMonth, isFirst = props.isFirst, isLast = props.isLast;
        var context = useDayPicker();
        var classNames = context.classNames, numberOfMonths = context.numberOfMonths, disableNavigation = context.disableNavigation, styles = context.styles, captionLayout = context.captionLayout, onMonthChange = context.onMonthChange, CaptionLabel = context.components.CaptionLabel;
        var _a = useNavigation(), previousMonth = _a.previousMonth, nextMonth = _a.nextMonth, setMonth = _a.setMonth;
        var handlePreviousClick = function (e) {
            if (!previousMonth)
                return;
            setMonth(previousMonth);
            onMonthChange === null || onMonthChange === void 0 ? void 0 : onMonthChange(previousMonth);
        };
        var handleNextClick = function (e) {
            if (!nextMonth)
                return;
            setMonth(nextMonth);
            onMonthChange === null || onMonthChange === void 0 ? void 0 : onMonthChange(nextMonth);
        };
        var handleMonthChange = function (newMonth) {
            setMonth(newMonth);
            onMonthChange === null || onMonthChange === void 0 ? void 0 : onMonthChange(newMonth);
        };
        var captionLabel = React.createElement(CaptionLabel, { displayMonth: displayMonth });
        return (React.createElement("div", { className: classNames[exports.UIElement.Caption], style: styles[exports.UIElement.Caption] }, disableNavigation ? (captionLabel) : (React.createElement(React.Fragment, null, captionLayout === 'dropdown' ? (React.createElement("div", { className: classNames[exports.UIElement.CaptionDropdowns], style: styles[exports.UIElement.CaptionDropdowns] },
            React.createElement(MonthsDropdown, { onChange: handleMonthChange, displayMonth: displayMonth }),
            React.createElement(YearsDropdown, { onChange: handleMonthChange, displayMonth: displayMonth }))) : (React.createElement(React.Fragment, null,
            React.createElement(CaptionLabel, { displayMonth: displayMonth }),
            React.createElement(Navigation, { displayMonth: displayMonth, hideNext: numberOfMonths > 1 && !isLast, hidePrevious: numberOfMonths > 1 && !isFirst, disableNext: !nextMonth, disablePrevious: !previousMonth, onPreviousClick: handlePreviousClick, onNextClick: handleNextClick })))))));
    }

    function CaptionLabel(props) {
        var _a = useDayPicker(), locale = _a.locale, classNames = _a.classNames, styles = _a.styles, formatCaption = _a.formatters.formatCaption;
        return (React__default['default'].createElement("div", { key: "caption", className: classNames[exports.UIElement.CaptionLabel], style: styles[exports.UIElement.CaptionLabel], "aria-live": "polite" }, formatCaption(props.displayMonth, { locale: locale })));
    }

    /**
     * The focus context holds values and method to handle the focused day via
     * keyboard navigation.
     */
    var FocusContext = React.createContext(undefined);
    /** Enable the FocusContext to handle the focus when navigating via keyboard. */
    var FocusProvider = function (_a) {
        var children = _a.children;
        var _b = React.useState(), focusedDay = _b[0], setDay = _b[1];
        var _c = useNavigation(), setMonth = _c.setMonth, displayMonths = _c.displayMonths;
        var _d = useDayPicker(), modifiers = _d.modifiers, numberOfMonths = _d.numberOfMonths;
        var blur = function () { return setDay(undefined); };
        var focus = function (date) { return setDay(date); };
        var switchMonth = function (date, offset) {
            if (displayMonths.some(function (m) { return dateFns.isSameMonth(date, m); }))
                return;
            if (offset < 0) {
                setMonth(dateFns.addMonths(date, 1 + offset));
            }
            else {
                setMonth(date);
            }
        };
        var isFocusable = function (day) {
            if (isMatch(day, modifiers.disabled))
                return false;
            return true;
        };
        var focusDayBefore = function () {
            if (!focusedDay)
                return;
            var before = dateFns.addDays(focusedDay, -1);
            if (!isFocusable(before))
                return;
            setDay(before);
            switchMonth(before, numberOfMonths * -1);
        };
        var focusDayAfter = function () {
            if (!focusedDay)
                return;
            var after = dateFns.addDays(focusedDay, 1);
            if (!isFocusable(after))
                return;
            setDay(after);
            switchMonth(after, numberOfMonths);
        };
        var focusDayUp = function () {
            if (!focusedDay)
                return;
            var up = dateFns.addWeeks(focusedDay, -1);
            if (!isFocusable(up))
                return;
            setDay(up);
            switchMonth(up, numberOfMonths * -1);
        };
        var focusDayDown = function () {
            if (!focusedDay)
                return;
            var down = dateFns.addWeeks(focusedDay, 1);
            if (!isFocusable(down))
                return;
            setDay(down);
            switchMonth(down, numberOfMonths);
        };
        var setters = {
            blur: blur,
            focus: focus,
            focusDayAfter: focusDayAfter,
            focusDayBefore: focusDayBefore,
            focusDayUp: focusDayUp,
            focusDayDown: focusDayDown
        };
        return (React.createElement(FocusContext.Provider, { value: [focusedDay, setters] }, children));
    };

    /** Return the context for handling focus via keyboard. */
    function useFocus() {
        var context = React.useContext(FocusContext);
        if (!context) {
            throw new Error('useFocus must be used within a FocusProvider');
        }
        return context;
    }

    function createHandlers(date, modifierStatus, context) {
        var onKeyUp = function (e) {
            var _a;
            (_a = context.onDayKeyUp) === null || _a === void 0 ? void 0 : _a.call(context, date, modifierStatus, e);
        };
        var onMouseEnter = function (e) {
            var _a;
            (_a = context.onDayMouseEnter) === null || _a === void 0 ? void 0 : _a.call(context, date, modifierStatus, e);
        };
        var onMouseLeave = function (e) {
            var _a;
            (_a = context.onDayMouseLeave) === null || _a === void 0 ? void 0 : _a.call(context, date, modifierStatus, e);
        };
        var onTouchCancel = function (e) {
            var _a;
            (_a = context.onDayTouchCancel) === null || _a === void 0 ? void 0 : _a.call(context, date, modifierStatus, e);
        };
        var onTouchEnd = function (e) {
            var _a;
            (_a = context.onDayTouchEnd) === null || _a === void 0 ? void 0 : _a.call(context, date, modifierStatus, e);
        };
        var onTouchMove = function (e) {
            var _a;
            (_a = context.onDayTouchMove) === null || _a === void 0 ? void 0 : _a.call(context, date, modifierStatus, e);
        };
        var onTouchStart = function (e) {
            var _a;
            (_a = context.onDayTouchStart) === null || _a === void 0 ? void 0 : _a.call(context, date, modifierStatus, e);
        };
        return {
            onKeyUp: onKeyUp,
            onMouseEnter: onMouseEnter,
            onMouseLeave: onMouseLeave,
            onTouchCancel: onTouchCancel,
            onTouchEnd: onTouchEnd,
            onTouchMove: onTouchMove,
            onTouchStart: onTouchStart
        };
    }

    /**
     * Render the content of a date cell, as a button or span element according to
     * its modifiers. Attaches the event handlers from DayPicker context, and manage the
     * focused date.
     */
    function Day(props) {
        var buttonRef = React.useRef(null);
        var date = props.date, displayMonth = props.displayMonth;
        var context = useDayPicker();
        var selection = useSelection();
        var modifierStatus = useModifiers(date);
        var _a = useFocus(), focusedDay = _a[0], _b = _a[1], focusDayAfter = _b.focusDayAfter, focusDayBefore = _b.focusDayBefore, focusDayDown = _b.focusDayDown, focusDayUp = _b.focusDayUp, blur = _b.blur, focus = _b.focus;
        var classNames = context.classNames, labelDay = context.labels.labelDay, DayContent = context.components.DayContent, locale = context.locale, modifierClassNames = context.modifierClassNames, modifierPrefix = context.modifierPrefix, modifierStyles = context.modifierStyles, showOutsideDays = context.showOutsideDays, styles = context.styles, onDayClick = context.onDayClick, mode = context.mode;
        React.useEffect(function () {
            var _a;
            if (!focusedDay)
                return;
            if (dateFns.isSameDay(focusedDay, date)) {
                (_a = buttonRef.current) === null || _a === void 0 ? void 0 : _a.focus();
            }
        }, [focusedDay]);
        if (modifierStatus.hidden)
            return React.createElement("span", null);
        var ariaLabel = labelDay(date, modifierStatus, { locale: locale });
        var ariaPressed = modifierStatus.selected;
        var handleClick = function (e) {
            var _a, _b;
            if (mode !== 'uncontrolled') {
                (_b = (_a = selection[mode]).onDayClick) === null || _b === void 0 ? void 0 : _b.call(_a, date, modifierStatus, e);
            }
            onDayClick === null || onDayClick === void 0 ? void 0 : onDayClick(date, modifierStatus, e);
        };
        var handleFocus = function (e) {
            focus(date);
        };
        var handleBlur = function (e) {
            blur();
        };
        var handleKeyDown = function (e) {
            switch (e.key) {
                case exports.KeyCode.ArrowLeft:
                    e.preventDefault();
                    e.stopPropagation();
                    focusDayBefore();
                    break;
                case exports.KeyCode.ArrowRight:
                    e.preventDefault();
                    e.stopPropagation();
                    focusDayAfter();
                    break;
                case exports.KeyCode.ArrowDown:
                    e.preventDefault();
                    e.stopPropagation();
                    focusDayDown();
                    break;
                case exports.KeyCode.ArrowUp:
                    e.preventDefault();
                    e.stopPropagation();
                    focusDayUp();
                    break;
            }
        };
        var otherEventHandlers = createHandlers(date, modifierStatus, context);
        var buttonClassNames = [classNames[exports.UIElement.Day]];
        var isOutside = !dateFns.isSameMonth(date, displayMonth);
        if (isOutside) {
            buttonClassNames.push(classNames[exports.UIElement.Outside]);
        }
        var isToday = dateFns.isSameDay(date, context.today);
        if (isToday) {
            buttonClassNames.push(classNames[exports.UIElement.Today]);
        }
        Object.keys(modifierStatus)
            .filter(function (modifier) { return Boolean(modifierStatus[modifier]); })
            .forEach(function (modifier) {
            if (modifierClassNames[modifier]) {
                buttonClassNames.push(modifierClassNames[modifier]);
            }
            else {
                buttonClassNames.push("" + modifierPrefix + modifier);
            }
        });
        var style = __assign({}, styles[exports.UIElement.Day]);
        if (styles) {
            Object.keys(modifierStatus).forEach(function (modifier) { return (style = __assign(__assign({}, style), styles[modifier])); });
        }
        if (modifierStyles) {
            Object.keys(modifierStatus).forEach(function (modifier) { return (style = __assign(__assign({}, style), modifierStyles[modifier])); });
        }
        var dayContent = (React.createElement(DayContent, { date: date, modifiers: modifierStatus, showOutsideDays: showOutsideDays, outside: isOutside, format: formatDay, locale: locale, displayMonth: displayMonth }));
        var className = buttonClassNames.join(' ');
        if (modifierStatus.disabled || isOutside) {
            return (React.createElement("span", { style: style, className: className }, dayContent));
        }
        var tabIndex = 0;
        // When a day is focused disable tab indexes in the other days
        if (focusedDay && !dateFns.isSameDay(focusedDay, date))
            tabIndex = -1;
        return (React.createElement("button", __assign({ ref: buttonRef, "aria-label": ariaLabel, "aria-pressed": ariaPressed, style: style, className: className, tabIndex: tabIndex, onClick: handleClick, onFocus: handleFocus, onBlur: handleBlur, onKeyDown: handleKeyDown }, otherEventHandlers), dayContent));
    }

    /**
     * Render the content of the day cell.
     */
    function DayContent(props) {
        if (props.outside && !props.showOutsideDays)
            return React__default['default'].createElement(React__default['default'].Fragment, null);
        if (props.modifiers.hidden)
            return React__default['default'].createElement(React__default['default'].Fragment, null);
        return React__default['default'].createElement(React__default['default'].Fragment, null, props.format(props.date, { locale: props.locale }));
    }

    /**
     * Render the date picker component.
     */
    function DayPicker(props) {
        return (React__default['default'].createElement(DayPickerProvider, { initialProps: props },
            React__default['default'].createElement(NavigationProvider, null,
                React__default['default'].createElement(SelectionProvider, null,
                    React__default['default'].createElement(FocusProvider, null,
                        React__default['default'].createElement(Root, null))))));
    }

    /**
     * Render a dropdown component (basically, a styled HTMLSelectElement).
     */
    function Dropdown(props) {
        var onChange = props.onChange, value = props.value, children = props.children, caption = props.caption, className = props.className, style = props.style;
        var _a = useDayPicker(), classNames = _a.classNames, styles = _a.styles, IconDropdown = _a.components.IconDropdown;
        return (React.createElement("div", { className: className, style: style },
            React.createElement("select", { className: classNames[exports.UIElement.Dropdown], style: styles[exports.UIElement.Dropdown], value: value, onChange: onChange }, children),
            React.createElement("div", { className: classNames[exports.UIElement.CaptionLabel], style: styles[exports.UIElement.CaptionLabel], "aria-hidden": "true" },
                caption,
                React.createElement(IconDropdown, { className: classNames[exports.UIElement.DropdownIcon], style: styles[exports.UIElement.DropdownIcon] }))));
    }

    /**
     * Render the Footer component (empty as default).
     */
    function Footer() {
        var footer = useDayPicker().footer;
        return (React__default['default'].createElement("tfoot", null,
            React__default['default'].createElement("tr", null,
                React__default['default'].createElement("td", { colSpan: 8 }, footer))));
    }

    /**
     * Render the Head component - i.e. the table head with the weekday names.
     */
    function Head() {
        var _a = useDayPicker(), classNames = _a.classNames, styles = _a.styles, showWeekNumber = _a.showWeekNumber, locale = _a.locale, formatWeekdayName = _a.formatters.formatWeekdayName, labelWeekday = _a.labels.labelWeekday, weekdays = _a.weekdays;
        return (React__default['default'].createElement("thead", { style: styles[exports.UIElement.Head], className: classNames[exports.UIElement.Head] },
            React__default['default'].createElement("tr", { style: styles[exports.UIElement.HeadRow], className: classNames[exports.UIElement.HeadRow] },
                showWeekNumber && (React__default['default'].createElement("th", { style: styles[exports.UIElement.HeadCell], className: classNames[exports.UIElement.HeadCell] })),
                weekdays.map(function (weekday, i) { return (React__default['default'].createElement("th", { key: i, scope: "col", className: classNames[exports.UIElement.HeadCell], style: styles[exports.UIElement.HeadCell], "aria-label": labelWeekday(weekday, { locale: locale }) }, formatWeekdayName(weekday, { locale: locale }))); }))));
    }

    /**
     * Render the icon in the styled drop-down.
     */
    function IconDropdown(props) {
        return (React__default['default'].createElement("svg", __assign({ width: "8px", height: "8px", viewBox: "0 0 120 120" }, props),
            React__default['default'].createElement("path", { d: "M4.22182541,48.2218254 C8.44222828,44.0014225 15.2388494,43.9273804 19.5496459,47.9996989 L19.7781746,48.2218254 L60,88.443 L100.221825,48.2218254 C104.442228,44.0014225 111.238849,43.9273804 115.549646,47.9996989 L115.778175,48.2218254 C119.998577,52.4422283 120.07262,59.2388494 116.000301,63.5496459 L115.778175,63.7781746 L67.7781746,111.778175 C63.5577717,115.998577 56.7611506,116.07262 52.4503541,112.000301 L52.2218254,111.778175 L4.22182541,63.7781746 C-0.0739418023,59.4824074 -0.0739418023,52.5175926 4.22182541,48.2218254 Z", fill: "currentColor", fillRule: "nonzero" })));
    }

    /**
     * Render the "next month" button in the navigation.
     */
    function IconNext(props) {
        return (React.createElement("svg", __assign({ width: "16px", height: "16px", viewBox: "0 0 120 120" }, props),
            React.createElement("path", { d: "M49.8040405,3.34314575 C46.6798462,0.218951416 41.6145263,0.218951416 38.490332,3.34314575 C35.4326099,6.40086786 35.367552,11.3179931 38.2951583,14.4548388 L38.490332,14.6568542 L83.8333725,60 L38.490332,105.343146 C35.4326099,108.400868 35.367552,113.317993 38.2951583,116.454839 L38.490332,116.656854 C41.5480541,119.714576 46.4651794,119.779634 49.602025,116.852028 L49.8040405,116.656854 L100.804041,65.6568542 C103.861763,62.5991321 103.926821,57.6820069 100.999214,54.5451612 L100.804041,54.3431458 L49.8040405,3.34314575 Z", fill: "currentColor" })));
    }

    /**
     * Render the "previous month" button in the navigation.
     */
    function IconPrevious(props) {
        return (React.createElement(React.Fragment, null,
            React.createElement("svg", __assign({ width: "16px", height: "16px", viewBox: "0 0 120 120" }, props),
                React.createElement("path", { d: "M69.490332,3.34314575 C72.6145263,0.218951416 77.6798462,0.218951416 80.8040405,3.34314575 C83.8617626,6.40086786 83.9268205,11.3179931 80.9992143,14.4548388 L80.8040405,14.6568542 L35.461,60 L80.8040405,105.343146 C83.8617626,108.400868 83.9268205,113.317993 80.9992143,116.454839 L80.8040405,116.656854 C77.7463184,119.714576 72.8291931,119.779634 69.6923475,116.852028 L69.490332,116.656854 L18.490332,65.6568542 C15.4326099,62.5991321 15.367552,57.6820069 18.2951583,54.5451612 L18.490332,54.3431458 L69.490332,3.34314575 Z", fill: "currentColor", fillRule: "nonzero" }))));
    }

    /**
     * Render the dropdown to navigate between months.
     */
    function MonthsDropdown(props) {
        var displayMonth = props.displayMonth;
        var _a = useDayPicker(), fromDate = _a.fromDate, toDate = _a.toDate, styles = _a.styles, locale = _a.locale, formatMonthCaption = _a.formatters.formatMonthCaption, classNames = _a.classNames, Dropdown = _a.components.Dropdown;
        var dropdownMonths = [];
        if (fromDate && toDate) {
            if (dateFns.isSameYear(fromDate, toDate)) {
                // only display the months included in the range
                for (var month = fromDate.getMonth(); month <= toDate.getMonth(); month++) {
                    dropdownMonths.push(dateFns.setMonth(dateFns.startOfMonth(fromDate), month));
                }
            }
            else {
                // display all the 12 months
                for (var month = 0; month <= 11; month++) {
                    var anyDate = new Date(); // any date is OK, we just need the year
                    dropdownMonths.push(dateFns.setMonth(dateFns.startOfMonth(anyDate), month));
                }
            }
        }
        var handleChange = function (e) {
            var newMonth = dateFns.setMonth(new Date(displayMonth), Number(e.target.value));
            props.onChange(newMonth);
        };
        return (React.createElement(Dropdown, { className: classNames[exports.UIElement.DropdownMonth], style: styles[exports.UIElement.DropdownMonth], onChange: handleChange, value: displayMonth.getMonth(), caption: formatMonthCaption(displayMonth, { locale: locale }) }, dropdownMonths.map(function (m) { return (React.createElement("option", { key: m.getMonth(), value: m.getMonth() }, formatMonthCaption(m, { locale: locale }))); })));
    }

    /** A component rendering the navigation buttons or the drop-downs. */
    function Navigation(props) {
        var _a;
        var _b = useDayPicker(), dir = _b.dir, locale = _b.locale, classNames = _b.classNames, styles = _b.styles, _c = _b.labels, labelPrevious = _c.labelPrevious, labelNext = _c.labelNext, _d = _b.components, IconNext = _d.IconNext, IconPrevious = _d.IconPrevious;
        var onPreviousClick = props.onPreviousClick, onNextClick = props.onNextClick;
        if (dir === 'rtl') {
            _a = [onPreviousClick, onNextClick], onNextClick = _a[0], onPreviousClick = _a[1];
        }
        var previousLabel = labelPrevious(props.displayMonth, { locale: locale });
        var previousClassName = [
            classNames[exports.UIElement.NavButton],
            classNames[exports.UIElement.NavButtonPrev]
        ].join(' ');
        var nextLabel = labelNext(props.displayMonth, { locale: locale });
        var nextClassName = [
            classNames[exports.UIElement.NavButton],
            classNames[exports.UIElement.NavButtonPrev]
        ].join(' ');
        var previousButton = (React__default['default'].createElement("button", { key: "prev", "aria-label": previousLabel, className: previousClassName, style: styles[exports.UIElement.NavButtonPrev], disabled: props.disablePrevious, onClick: onPreviousClick },
            React__default['default'].createElement(IconPrevious, { className: classNames[exports.UIElement.NavIcon], style: styles[exports.UIElement.NavIcon] })));
        var nextButton = (React__default['default'].createElement("button", { key: "next", "aria-label": nextLabel, className: nextClassName, disabled: props.disableNext, onClick: onNextClick, style: styles[exports.UIElement.NavButtonNext] },
            React__default['default'].createElement(IconNext, { className: classNames[exports.UIElement.NavIcon], style: styles[exports.UIElement.NavIcon] })));
        if (props.disableNext && props.disablePrevious) {
            return React__default['default'].createElement(React__default['default'].Fragment, null);
        }
        return (React__default['default'].createElement("div", { className: classNames[exports.UIElement.Nav], style: styles[exports.UIElement.Nav] },
            !props.hidePrevious && previousButton,
            !props.hideNext && nextButton));
    }

    /**
     * Render the container with the months and their captions. The number of months
     * rendered depends by the `numberOfMonths` prop.
     */
    function Root(props) {
        var className = props.className, style = props.style;
        var _a = useDayPicker(), dir = _a.dir, classNames = _a.classNames, styles = _a.styles, numberOfMonths = _a.numberOfMonths, Caption = _a.components.Caption;
        var displayMonths = useNavigation().displayMonths;
        var rootClassNames = [classNames[exports.UIElement.Root]];
        if (numberOfMonths > 1) {
            rootClassNames.push(classNames[exports.UIElement.RootMultipleMonths]);
        }
        if (className)
            rootClassNames.concat(className.split(' '));
        var renderMonth = function (displayMonth, displayIndex) {
            var _a;
            var className = [classNames[exports.UIElement.Month]];
            var style = __assign({}, styles[exports.UIElement.Month]);
            var isFirst = numberOfMonths > 1 && displayIndex === 0;
            var isLast = numberOfMonths > 1 && displayIndex === displayMonths.length - 1;
            if (dir === 'rtl')
                _a = [isFirst, isLast], isLast = _a[0], isFirst = _a[1];
            var isBetween = numberOfMonths > 1 && !isFirst && !isLast;
            if (isFirst) {
                className.push(classNames[exports.UIElement.CaptionFirst]);
                Object.assign(style, styles[exports.UIElement.CaptionFirst]);
            }
            if (isLast)
                className.push(classNames[exports.UIElement.CaptionLast]);
            if (isBetween)
                className.push(classNames[exports.UIElement.CaptionBetween]);
            return (React.createElement("div", { key: displayIndex, className: className.join(' '), style: style },
                React.createElement(Caption, { displayMonth: displayMonth, displayIndex: displayIndex, isFirst: isFirst, isLast: isLast, isBetween: isBetween }),
                React.createElement(Table, { displayMonth: displayMonth })));
        };
        return (React.createElement("div", { className: rootClassNames.join(' '), style: __assign(__assign({}, styles[exports.UIElement.Root]), style), dir: dir },
            React.createElement("div", { className: classNames[exports.UIElement.Months], style: styles[exports.UIElement.Months] }, displayMonths.map(renderMonth))));
    }

    /**
     * Render a row in the calendar, with the days and optionally the week number.
     */
    function Row(props) {
        var _a = useDayPicker(), styles = _a.styles, classNames = _a.classNames, showWeekNumber = _a.showWeekNumber, _b = _a.components, Day = _b.Day, WeekNumber = _b.WeekNumber;
        var weekNumberCell;
        if (showWeekNumber) {
            weekNumberCell = (React.createElement("th", { className: classNames[exports.UIElement.RowHead], style: styles[exports.UIElement.RowHead] },
                React.createElement(WeekNumber, { number: props.weekNumber, dates: props.dates })));
        }
        return (React.createElement("tr", { className: classNames[exports.UIElement.Row], style: styles[exports.UIElement.Row] },
            weekNumberCell,
            props.dates.map(function (date) { return (React.createElement("td", { className: classNames[exports.UIElement.Cell], style: styles[exports.UIElement.Cell], key: dateFns.getUnixTime(date) },
                React.createElement(Day, { displayMonth: props.displayMonth, date: date }))); })));
    }

    function getOutsideEndDays(month, options) {
        var days = [];
        var lastDayOfWeek = dateFns.endOfWeek(month, options);
        var endDiff = dateFns.differenceInCalendarDays(lastDayOfWeek, month);
        for (var i = 1; i <= endDiff; i++) {
            var dayDate = dateFns.addDays(month, i);
            days.push(dayDate);
        }
        return days;
    }

    function getOutsideStartDays(month, options) {
        var days = [];
        var firstDayOfWeek = dateFns.startOfWeek(month, options);
        var startDiff = dateFns.differenceInCalendarDays(month, firstDayOfWeek);
        for (var i = 0; i < startDiff; i++) {
            var newDay = dateFns.addDays(firstDayOfWeek, i);
            days.push(newDay);
        }
        return days;
    }

    /**
     * Return the weeks belonging to the given month.
     */
    function getWeeks(month, _a) {
        var locale = _a.locale, fixedWeeks = _a.fixedWeeks;
        var monthStart = dateFns.startOfMonth(month);
        var monthEnd = dateFns.endOfMonth(month);
        var diff = dateFns.differenceInCalendarDays(monthEnd, monthStart);
        var weeks = {};
        var lastWeekStr = '';
        for (var i = 0; i <= diff; i++) {
            var date = dateFns.addDays(monthStart, i);
            var week = dateFns.getWeek(date, { locale: locale });
            if (week === 1 && dateFns.getMonth(date) === 11) {
                week = 53;
            }
            var weekStr = week.toString();
            if (!weeks[weekStr]) {
                var startDays = getOutsideStartDays(date, { locale: locale });
                // Create a new week by adding outside start days
                weeks[weekStr] = startDays;
            }
            weeks[weekStr].push(date);
            lastWeekStr = weekStr;
        }
        var lastWeek = weeks[lastWeekStr];
        var lastDay = lastWeek[lastWeek.length - 1];
        var endDays = getOutsideEndDays(lastDay, { locale: locale });
        weeks[lastWeekStr] = lastWeek.concat(endDays);
        // Add extra weeks to the month, up to 6 weeks
        if (fixedWeeks) {
            lastWeek = weeks[lastWeekStr];
            var lastWeekDate = lastWeek[lastWeek.length - 1];
            var weeksInMonth = dateFns.getWeeksInMonth(month, { locale: locale });
            if (weeksInMonth < 6) {
                var diffDays = dateFns.differenceInCalendarDays(dateFns.addWeeks(lastWeekDate, 6 - weeksInMonth), lastWeekDate);
                for (var i = 0; i < diffDays; i++) {
                    var date = dateFns.addDays(lastWeekDate, i + 1);
                    var week = dateFns.getWeek(date, { locale: locale });
                    if (week === 1 && dateFns.getMonth(month) === 11) {
                        week = 53;
                    }
                    if (!weeks[week]) {
                        weeks[week] = [];
                    }
                    weeks[week.toString()].push(date);
                }
            }
        }
        return weeks;
    }

    /**
     * Render the table with the calendar.
     */
    function Table(props) {
        var _a = useDayPicker(), locale = _a.locale, classNames = _a.classNames, styles = _a.styles, hideHead = _a.hideHead, fixedWeeks = _a.fixedWeeks, _b = _a.components, Row = _b.Row, Footer = _b.Footer;
        var weeks = getWeeks(props.displayMonth, { locale: locale, fixedWeeks: fixedWeeks });
        return (React.createElement("table", { className: classNames[exports.UIElement.Table], style: styles[exports.UIElement.Table] },
            !hideHead && React.createElement(Head, null),
            React.createElement("tbody", { className: classNames[exports.UIElement.TBody], style: styles[exports.UIElement.TBody] }, Object.keys(weeks).map(function (weekNumber) { return (React.createElement(Row, { displayMonth: props.displayMonth, key: weekNumber, dates: weeks[weekNumber], weekNumber: Number(weekNumber) })); })),
            React.createElement(Footer, null)));
    }

    /**
     * Render the week number element. If `onWeekNumberClick` is passed to DayPicker, it
     * renders a button, otherwise a span element.
     */
    function WeekNumber(props) {
        var weekNumber = props.number, dates = props.dates;
        var _a = useDayPicker(), onWeekNumberClick = _a.onWeekNumberClick, styles = _a.styles, classNames = _a.classNames, locale = _a.locale, labelWeekNumber = _a.labels.labelWeekNumber, formatWeekNumber = _a.formatters.formatWeekNumber;
        var label = labelWeekNumber(Number(weekNumber), { locale: locale });
        var handleClick = function (e) {
            onWeekNumberClick === null || onWeekNumberClick === void 0 ? void 0 : onWeekNumberClick(weekNumber, dates, e);
        };
        var Component = onWeekNumberClick ? 'button' : 'span';
        return (React__default['default'].createElement(Component, { "aria-label": label, className: classNames[exports.UIElement.WeekNumber], style: styles[exports.UIElement.WeekNumber], onClick: handleClick }, formatWeekNumber(Number(weekNumber), { locale: locale })));
    }

    /**
     * Render a dropdown to change the year. Take in account the `nav.fromDate` and
     * `toDate` from context.
     */
    function YearsDropdown(props) {
        var displayMonth = props.displayMonth;
        var _a = useDayPicker(), fromDate = _a.fromDate, toDate = _a.toDate, locale = _a.locale, styles = _a.styles, classNames = _a.classNames, Dropdown = _a.components.Dropdown, formatYearCaption = _a.formatters.formatYearCaption;
        var years = [];
        if (fromDate && toDate) {
            var fromYear = fromDate.getFullYear();
            var toYear = toDate.getFullYear();
            for (var year = fromYear; year <= toYear; year++) {
                years.push(dateFns.setYear(dateFns.startOfYear(new Date()), year));
            }
        }
        var handleChange = function (e) {
            var newMonth = dateFns.setYear(new Date(displayMonth), Number(e.target.value));
            props.onChange(newMonth);
        };
        return (React.createElement(Dropdown, { className: classNames[exports.UIElement.DropdownMonth], style: styles[exports.UIElement.DropdownMonth], onChange: handleChange, value: displayMonth.getFullYear(), caption: formatYearCaption(displayMonth, { locale: locale }) }, years.map(function (year) { return (React.createElement("option", { key: year.getFullYear(), value: year.getFullYear() }, formatYearCaption(year, { locale: locale }))); })));
    }

    exports.Caption = Caption;
    exports.CaptionLabel = CaptionLabel;
    exports.Day = Day;
    exports.DayContent = DayContent;
    exports.DayPicker = DayPicker;
    exports.DayPickerContext = DayPickerContext;
    exports.DayPickerProvider = DayPickerProvider;
    exports.Dropdown = Dropdown;
    exports.Footer = Footer;
    exports.Head = Head;
    exports.IconDropdown = IconDropdown;
    exports.IconNext = IconNext;
    exports.IconPrevious = IconPrevious;
    exports.MonthsDropdown = MonthsDropdown;
    exports.Navigation = Navigation;
    exports.NavigationContext = NavigationContext;
    exports.NavigationProvider = NavigationProvider;
    exports.Root = Root;
    exports.Row = Row;
    exports.SelectionContext = SelectionContext;
    exports.SelectionProvider = SelectionProvider;
    exports.Table = Table;
    exports.WeekNumber = WeekNumber;
    exports.YearsDropdown = YearsDropdown;
    exports.useDayPicker = useDayPicker;
    exports.useInput = useInput;
    exports.useModifiers = useModifiers;
    exports.useNavigation = useNavigation;
    exports.useSelection = useSelection;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=index.umd.js.map
